StorageEngine = function() {
    var initialized = false;
    var initializedObjectStores = [];
    
    function running(type,errorCallback) {
        if(!initialized){
            errorCallback('storage_api_not_initialized',
                          'The storage engine has not been initialized');
            return false;
        } else if (!initializedObjectStores[type]) {
            errorCallback('store_not_initialized',
                          'The object store '+type+' is not initilaized');
            return false;
        }
        
        return true;
    }
    
    this.init = function(successCallback, errorCallback) {
        if (window.localStorage) {
            initialized = true;
            successCallback(null);
        } else {
            errorCallback('storage_api_not_supported',
                          'The web storage api is not supported.');
        }
    };
    
    this.initObjectStore = function(type, successCallback, errorCallback) {
        if(!initialized){
            errorCallback('storage_api_not_initialized',
                          'The storage engine has not been initialized');
            return;
        } else if (!localStorage.getItem(type)) {
            localStorage.setItem(type, JSON.stringify({}));
        }
        
        initializedObjectStores[type] = true;
        successCallback(null);
    };
    
    this.save = function(type, obj, successCallback, errorCallback){
        if(!running(type,errorCallback)) {
            return;
        }
        
        if (!obj.id) {
            obj.id = $.now();
        }
        
        var storageItem = JSON.parse(localStorage.getItem(type));
        storageItem[obj.id] = obj;
        
        localStorage.setItem(type, JSON.stringify(storageItem));
        successCallback(obj);
        
        
    };

    this.findAll = function(type, successCallback, errorCallback) {
        if(!running(type,errorCallback)) {
            return;
        }
        var result = [];
        var storedItem = JSON.parse(localStorage.getItem(type));
        $.each(storedItem,function(i,e){
            result.push(e);
        });
        successCallback(result);
    };
    
    this.delete = function(type, id, successCallback, errorCallback) {
        if(!running(type,errorCallback)) {
            return;
        }
        
        var storageItem = JSON.parse(localStorage.getItem(type));
        if (storageItem[id]) {
            delete storageItem[id];
            localStorage.setItem(type, JSON.stringify(storageItem));
            successCallback(id);
        } else {
            errorCallback("object_not_found", "The object requested could not be found");
        }
    };
    
    this.find = function(type, id, successCallback, errorCallback) {
        if(!running(type,errorCallback)) {
            return;
        }
        
        var storageItem = JSON.parse(localStorage.getItem(type));
        
        if (storageItem[id]) {
            successCallback(storageItem[id]);
        } else {
            errorCallback("object_not_found", "The object requested could not be found");
        }
    };
}