//$(document).ready(handler)
// same as $(handler)

// when the dom is ready
$(function() {
    // things to do once the page has loaded
    var storage = new StorageEngine();
    
    function errorlogger(errorCode, errorMessage) {
        alert(errorCode + ": " + errorMessage);
    }
    
    storage.init(function() {
        storage.initObjectStore('todo',function() {
            storage.findAll('todo', function(items) {
                // is there a table
                if($('#todo table').length === 0 && items.length > 0) {
                    // there is no table yet, so add it
                    $('#todo').append('<table class="table" id="todo-table"><thead><tr><th>Description</th><th>Priority</th><th>Due Date</th><th></th></tr></thead><tbody id="todo-body"></tbody></table>');
                }
                
                $.each(items, function(id,itm) {
                    var eventHTML = $('<tr id="'+itm.id+'"><td>'+itm.desc+'</td><td>'+itm.priority+'</td><td class="todo-date">'+itm.due+'</td><td><span class="glyphicon glyphicon-remove todo-remove"></span></td></tr>');
                    $("#todo-body").append(eventHTML);
                });
            }, errorlogger);
        }, errorlogger);
    }, errorlogger);
    
    
    // handler for when the button is clicked
    $("#add").click(function(evt){
        evt.preventDefault();
        
        // validate the form input
        var desc = $('#description').val();
        if(desc === "") {
            $('#description').parent().addClass("has-error");
            alert("You have an empty description!");
            return;
        }
        
        var due = $('#due').val();
        if(due === "" || isNaN((new Date(due)).valueOf()) ) {
            $('#due').parent().addClass("has-error");
            alert("You have an invalid due date");
            return;
        }
        
        var priority = $('#priority').val();
        
        var todo = {
            desc: desc,
            due : new Date(due),
            priority: priority
        };
        
        // is there a table
        if($('#todo table').length === 0) {
            // there is no table yet, so add it
            $('#todo').append('<table class="table" id="todo-table"><thead><tr><th>Description</th><th>Priority</th><th>Due Date</th><th></th></tr></thead><tbody id="todo-body"></tbody></table>');
        }
        // add the event
        
        storage.save('todo',todo,function(obj){
            var eventHTML = $('<tr><td>'+desc+'</td><td>'+priority+'</td><td class="todo-date">'+due+'</td><td><span class="glyphicon glyphicon-remove todo-remove"></span></td></tr>');
            eventHTML.attr('id',obj.id);
            $("#todo-body").append(eventHTML);
            
            //empty the submission form
            $('#description').val("");
            $('#due').val("");
            $('#priority').val("normal");
        },errorlogger);
        
        $('.has-error').removeClass("has-error");
        
    });
    
    $("#todo").on("click", "table .todo-remove", function(evt) {
        evt.preventDefault();
        
        storage.delete('todo',$(evt.target).parents('tr').attr('id'), function() {
            $(evt.target).parents('tr').remove();
            if($('#todo-body tr').length === 0 ) {
                $('#todo table').remove();
            }
        }, errorlogger);
        
    });
    
    setInterval(function() {
        $.each($(".todo-date"),function(index,element) {
            //for each date
            var date = new Date($(element).text());
            var curr = new Date();
            
            if(date < curr) {
                $(element).parents("tr").addClass("red");
            }
        });
    }, 60000);
});

window.applicationCache.addEventListener("updateready", function() {
    window.location.reload();
});