StorageEngine = function() {
    // constructor for the StorageEngine
    var initialized = false;
    var initializedObjectStores = [];
    
    this.init = function(successCallback, errorCallback) {
        // we need to check if the browser supports Web Storage
        if(window.localStorage) {
            initialized = true;
            successCallback(null);
        } else {
            errorCallback('storage_api_not_supported',
                          'The web storage api is not supported');
        }
    };
    
    this.initObjectStore = function(folder,successCallback, errorCallback) {
        // connected into a "folder" in the local storage
        if(!initialized) {
            errorCallback('storage_api_not_initialized',
                          'The storage enigne has not been initialized');
            return;
        } else if (!localStorage.getItem(folder)) {
            // the folder doesn't exist
            // create the folder
            localStorage.setItem(folder,JSON.stringify({}));
        }
        
        // we have the folder
        initializedObjectStores[folder] = true;
        successCallback(null);
    };
    
    // save an object into folder
    // we'll return the object if successful
    this.save = function(folder, obj, successCallback, errorCallback) {
        // initObjectStore[folder] must be true
        // must be initialized
        if(!initialized) {
            errorCallback('storage_api_not_initialized',
                          'The storage enigne has not been initialized');
            return;
        } else if (!initializedObjectStores[folder]) {
            errorCallback('store_not_initialized',
                          'The storage folder has not been initialized');
            return;
        }
        
        if(!obj.id) {
            obj.id = $.now();
        }
        
        // get folder content
        var folderContents = JSON.parse(localStorage.getItem(folder));
        
        // add object to folder
        folderContents[obj.id] = obj;
        
        // save back the folder with the new content
        localStorage.setItem(folder,JSON.stringify(folderContents));
        successCallback(obj);
    };
    
    // return an array of all the objects in the folder
    this.findAll = function(folder, successCallback, errorCallback) {
        // initObjectStore[folder] must be true
        // must be initialized
        if(!initialized) {
            errorCallback('storage_api_not_initialized',
                          'The storage enigne has not been initialized');
            return;
        } else if (!initializedObjectStores[folder]) {
            errorCallback('store_not_initialized',
                          'The storage folder has not been initialized');
            return;
        }
        
        // make an array to store the results to return
        var result = [];
        
        var folderContents = JSON.parse(localStorage.getItem(folder));
        
        // add each of the elements from the folder into the result array
        $.each(folderContents, function(index, object) {
            result.push(object);
        });
        
        successCallback(result);
    };
    
    // delete object with id "id" from folder
    // this.delete = function(folder, id, successCallback, errorCallback)
    // in JS, we can say "delete <key>" to remove an element from an object
    
    // get an individual item out of a folder by id
    // and return it to the successCallback
    // this.find = function(folder, id, successCallback, errorCallback)
}